@@ -1,6 +1,6 @@
 /*
  * activity.c: Define system activities available for sar/sadc.
- * (C) 1999-2019 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2020 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -1873,6 +1873,144 @@ struct activity softnet_act = {
 	.bitmap		= &cpu_bitmap
 };
 
+/* Pressure-stall CPU activity */
+struct activity psi_cpu_act = {
+	.id		= A_PSI_CPU,
+	.options	= AO_COLLECTED + AO_DETECTED,
+	.magic		= ACTIVITY_MAGIC_BASE,
+	.group		= G_DEFAULT,
+#ifdef SOURCE_SADC
+	.f_count_index	= 11,
+	.f_count2	= NULL,
+	.f_read		= wrap_read_psicpu,
+#endif
+#ifdef SOURCE_SAR
+	.f_print	= print_psicpu_stats,
+	.f_print_avg	= print_avg_psicpu_stats,
+#endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "%scpu-10;%scpu-60;%scpu-300;%scpu",
+#endif
+	.gtypes_nr	= {STATS_PSI_CPU_ULL, STATS_PSI_CPU_UL, STATS_PSI_CPU_U},
+	.ftypes_nr	= {0, 0, 0},
+#ifdef SOURCE_SADF
+	.f_render	= render_psicpu_stats,
+	.f_xml_print	= xml_print_psicpu_stats,
+	.f_json_print	= json_print_psicpu_stats,
+	.f_svg_print	= svg_print_psicpu_stats,
+	.f_raw_print	= raw_print_psicpu_stats,
+	.f_pcp_print	= pcp_print_psicpu_stats,
+	.f_count_new	= NULL,
+	.item_list	= NULL,
+	.desc		= "Pressure-stall CPU statistics",
+#endif
+	.name		= "A_PSI_CPU",
+	.item_list_sz	= 0,
+	.g_nr		= 2,
+	.nr_ini		= 1,
+	.nr2		= 1,
+	.nr_max		= 1,
+	.nr		= {1, 1, 1},
+	.nr_allocated	= 0,
+	.fsize		= STATS_PSI_CPU_SIZE,
+	.msize		= STATS_PSI_CPU_SIZE,
+	.opt_flags	= 0,
+	.buf		= {NULL, NULL, NULL},
+	.bitmap		= NULL
+};
+
+/* Pressure-stall I/O activity */
+struct activity psi_io_act = {
+	.id		= A_PSI_IO,
+	.options	= AO_COLLECTED + AO_DETECTED,
+	.magic		= ACTIVITY_MAGIC_BASE,
+	.group		= G_DEFAULT,
+#ifdef SOURCE_SADC
+	.f_count_index	= 11,
+	.f_count2	= NULL,
+	.f_read		= wrap_read_psiio,
+#endif
+#ifdef SOURCE_SAR
+	.f_print	= print_psiio_stats,
+	.f_print_avg	= print_avg_psiio_stats,
+#endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "%sio-10;%sio-60;%sio-300;%sio;%fio-10;%fio-60;%fio-300;%fio",
+#endif
+	.gtypes_nr	= {STATS_PSI_IO_ULL, STATS_PSI_IO_UL, STATS_PSI_IO_U},
+	.ftypes_nr	= {0, 0, 0},
+#ifdef SOURCE_SADF
+	.f_render	= render_psiio_stats,
+	.f_xml_print	= xml_print_psiio_stats,
+	.f_json_print	= json_print_psiio_stats,
+	.f_svg_print	= svg_print_psiio_stats,
+	.f_raw_print	= raw_print_psiio_stats,
+	.f_pcp_print	= pcp_print_psiio_stats,
+	.f_count_new	= NULL,
+	.item_list	= NULL,
+	.desc		= "Pressure-stall I/O statistics",
+#endif
+	.name		= "A_PSI_IO",
+	.item_list_sz	= 0,
+	.g_nr		= 4,
+	.nr_ini		= 1,
+	.nr2		= 1,
+	.nr_max		= 1,
+	.nr		= {1, 1, 1},
+	.nr_allocated	= 0,
+	.fsize		= STATS_PSI_IO_SIZE,
+	.msize		= STATS_PSI_IO_SIZE,
+	.opt_flags	= 0,
+	.buf		= {NULL, NULL, NULL},
+	.bitmap		= NULL
+};
+
+/* Pressure-stall memory activity */
+struct activity psi_mem_act = {
+	.id		= A_PSI_MEM,
+	.options	= AO_COLLECTED + AO_DETECTED + AO_CLOSE_MARKUP,
+	.magic		= ACTIVITY_MAGIC_BASE,
+	.group		= G_DEFAULT,
+#ifdef SOURCE_SADC
+	.f_count_index	= 11,
+	.f_count2	= NULL,
+	.f_read		= wrap_read_psimem,
+#endif
+#ifdef SOURCE_SAR
+	.f_print	= print_psimem_stats,
+	.f_print_avg	= print_avg_psimem_stats,
+#endif
+#if defined(SOURCE_SAR) || defined(SOURCE_SADF)
+	.hdr_line	= "%smem-10;%smem-60;%smem-300;%smem;%fmem-10;%fmem-60;%fmem-300;%fmem",
+#endif
+	.gtypes_nr	= {STATS_PSI_MEM_ULL, STATS_PSI_MEM_UL, STATS_PSI_MEM_U},
+	.ftypes_nr	= {0, 0, 0},
+#ifdef SOURCE_SADF
+	.f_render	= render_psimem_stats,
+	.f_xml_print	= xml_print_psimem_stats,
+	.f_json_print	= json_print_psimem_stats,
+	.f_svg_print	= svg_print_psimem_stats,
+	.f_raw_print	= raw_print_psimem_stats,
+	.f_pcp_print	= pcp_print_psimem_stats,
+	.f_count_new	= NULL,
+	.item_list	= NULL,
+	.desc		= "Pressure-stall memory statistics",
+#endif
+	.name		= "A_PSI_MEM",
+	.item_list_sz	= 0,
+	.g_nr		= 4,
+	.nr_ini		= 1,
+	.nr2		= 1,
+	.nr_max		= 1,
+	.nr		= {1, 1, 1},
+	.nr_allocated	= 0,
+	.fsize		= STATS_PSI_MEM_SIZE,
+	.msize		= STATS_PSI_MEM_SIZE,
+	.opt_flags	= 0,
+	.buf		= {NULL, NULL, NULL},
+	.bitmap		= NULL
+};
+
 #ifdef SOURCE_SADC
 /*
  * Array of functions used to count number of items.
@@ -1888,7 +2026,8 @@ __nr_t (*f_count[NR_F_COUNT]) (struct activity *) = {
 	wrap_get_in_nr,
 	wrap_get_usb_nr,
 	wrap_get_filesystem_nr,
-	wrap_get_fchost_nr
+	wrap_get_fchost_nr,
+	wrap_detect_psi
 };
 #endif
 
@@ -1937,7 +2076,12 @@ struct activity *act[NR_ACT] = {
 	&pwr_temp_act,
 	&pwr_in_act,
 	&pwr_wghfreq_act,
-	&pwr_usb_act,		/* AO_CLOSE_MARKUP */
+	&pwr_usb_act,	/* AO_CLOSE_MARKUP */
 	/* </power-management> */
-	&filesystem_act
+	&filesystem_act,
+	/* <psi> */
+	&psi_cpu_act,
+	&psi_io_act,
+	&psi_mem_act	/* AO_CLOSE_MARKUP */
+	/* </psi> */
 };
